/**
 * Created by Shaffer on 2017年8月29日 029.
 */
'use strict';
const crypto = require('crypto');
const rand = require('csprng');
const bcrypt = require('bcrypt');

const config = require('./config');


function passwordHash(plainPassword,salt=null) {
    //先将密码经过sha512，返回64位字符串
    const hasher = crypto.createHash("sha512");
    hasher.update(plainPassword);
    const password_nonsalt = hasher.digest('hex');
    // 如果salt不存在而且salt长度不等于8，则程序自动生成
    if(!salt || salt.length!==8){
        // csprng(bits, radix)，(41,36)返回长度为8的随机字符串
        // 算法详见源码
        salt = rand(41, 36);
    }

    const password_salted = salt.substr(0, 4) + password_nonsalt + salt.substr(4, 4);
    // Bcrypt算法 salt 强度为10
    const saltRounds = 10;
    const bcrypt_hash = bcrypt.hashSync(password_salted, saltRounds);
    // 最后再用aes加密一次
    // const hash = aes_encrypt(bcrypt_hash, app.config.GLOBAL_KEY);

    const hash = aes_encrypt(bcrypt_hash, config.get('core').GLOBAL_KEY);

    return {
        hash: hash,
        salt: salt
    };
}

function comparePassword(plainPassword, salt, aes_encrypted) {
    const bcrypt_hash = aes_decrypt(aes_encrypted, config.get('core').GLOBAL_KEY);
    const hasher = crypto.createHash("sha512");
    hasher.update(plainPassword);
    const password_nonsalt = hasher.digest('hex');
    const password_salted = salt.substr(0, 4) + password_nonsalt + salt.substr(4, 4);
    return bcrypt.compareSync(password_salted, bcrypt_hash);
}

function aes_encrypt(data, key) {
    var iv = "";
    var clearEncoding = 'utf8';
    var cipherEncoding = 'base64';
    var cipherChunks = [];
    var cipher = crypto.createCipheriv('aes-128-ecb', key, iv);
    cipher.setAutoPadding(true);
    cipherChunks.push(cipher.update(data, clearEncoding, cipherEncoding));
    cipherChunks.push(cipher.final(cipherEncoding));
    return cipherChunks.join('');
}

function aes_decrypt(data, key) {
    var iv = "";
    var clearEncoding = 'utf8';
    var cipherEncoding = 'base64';
    var cipherChunks = [];
    var decipher = crypto.createDecipheriv('aes-128-ecb', key, iv);
    decipher.setAutoPadding(true);
    cipherChunks.push(decipher.update(data, cipherEncoding, clearEncoding));
    cipherChunks.push(decipher.final(clearEncoding));
    return cipherChunks.join('');
}

function generateCookie(obj){
    const str = JSON.stringify(obj);
    // const base64Str = new Buffer(str).toString('base64');
    return aes_encrypt(str, config.get('core').GLOBAL_KEY);
}

function resolveCookie(str){
    const obj = aes_decrypt(str, config.get('core').GLOBAL_KEY);
    // const base64Str = new Buffer(str).toString('base64');
    return JSON.parse(obj);
}

module.exports = {
    passwordHash: passwordHash,
    comparePassword: comparePassword,
    aes_encrypt: aes_encrypt,
    aes_decrypt: aes_decrypt,
    generateCookie: generateCookie,
    resolveCookie: resolveCookie,
};
